﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS.SteadyState {
  [Item("RandomIntAssigner", "An operator which sets a random value to a parameter")]
  [StorableClass]
  public class RandomIntAssigner : SingleSuccessorOperator, IStochasticOperator {
    public LookupParameter<IntValue> LeftSideParameter {
      get { return (LookupParameter<IntValue>)Parameters["LeftSide"]; }
    }
    public ValueLookupParameter<IntValue> MinimumParameter {
      get { return (ValueLookupParameter<IntValue>)Parameters["Minimum"]; }
    }
    public ValueLookupParameter<IntValue> MaximumParameter {
      get { return (ValueLookupParameter<IntValue>)Parameters["Maximum"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }

    [StorableConstructor]
    private RandomIntAssigner(bool deserializing) : base(deserializing) { }
    private RandomIntAssigner(RandomIntAssigner original, Cloner cloner)
      : base(original, cloner) {
    }
    public RandomIntAssigner()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("LeftSide", "The parameter whose value gets assigned."));
      Parameters.Add(new ValueLookupParameter<IntValue>("Minimum", new IntValue(0)));
      Parameters.Add(new ValueLookupParameter<IntValue>("Maximum", new IntValue(10)));
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator to use."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RandomIntAssigner(this, cloner);
    }

    public override IOperation Apply() {
      var random = RandomParameter.ActualValue;
      int min = MinimumParameter.ActualValue.Value;
      int max = MaximumParameter.ActualValue.Value;
      LeftSideParameter.ActualValue = new IntValue(random.Next(min, max));
      return base.Apply();
    }
  }
}